import sys
import os

from time import sleep, time

from clr import * #pip install pythonnet
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))
import clr
from System.Text import StringBuilder


clr.AddReference('UsbDllWrap')
import clr
import Newport

class TLB6700:
    def refresh(self):
        """Connect to the laser; currently only supports a single laser"""
        self.npusb = Newport.USBComm.USB()
        self.npusb.OpenDevices(4106, True) #Product id = 4106 (dec) = 0x100a (hex)
        self.ndevices,self.keystrings = self.npusb.GetDeviceKeys('')
        if self.ndevices == 1:
            self.devicekey = self.keystrings[0] #Should be u'TLB-6700-LN SN1008'
            print(self.devicekey+' connected.')
            self.write('syst:mcon rem')
        else:
            print('{} laser devices detected.'.format(self.ndevices))
    
    def __init__(self):
        self.b = StringBuilder(64)
        self.refresh()

    #Basic I/O
    def query(self,text):
        """Query command used for everything"""
        self.b.Clear()
        self.npusb.Query(self.devicekey,text,self.b)
        return self.b.ToString()

    def write(self,text):
        """Returns error if command fails"""
        if text[-1] == '?':
            raise Exception('Query command sent.')
        if self.query(text) != u'OK':
            raise Exception('Command {} failed with response {}.'.format(text,self.b.ToString()))

    def msg(self,text):
        """Interprets communication type"""
        if text[-1] == '?':
            return self.query(text)
        else:
            self.write(text)

    #Set and read things
    def set_power(self,mW=''):
        """Set power in mW"""
        
        if self.query('sour:cpow?') != '1':
            self.query('sour:cpow 1')
            print('Switched to power control.')
            self.on()

        if isinstance(mW,float) or isinstance(mW,int):
            self.write('sour:pow:diod {}'.format(mW))
        else:
            return self.query('sour:pow:diod?')
    
    def read_power(self):
        """Return instantaneous power in mW"""
        return self.query('sens:pow:diod?')

    def set_current(self,mA=''):
        """Set current in mA"""

        if self.query('sour:cpow?') != '0':
            self.query('sour:cpow 0')
            print('Switched to current control.')
            self.on()

        if isinstance(mA,float) or isinstance(mA,int):
            self.write('sour:curr:diod {}'.format(mA))
        else:
            return self.query('sour:curr:diod?')

    def read_current(self):
        """Return instantaneous current in mA"""
        return self.query('sens:curr:diod?')

    def set_piezo(self,pc=''):
        """Set piezo as percentage"""
        if isinstance(pc,float) or isinstance(pc,int):
            self.write('sour:volt:piez {}'.format(pc))
        else:
            return self.query('sour:volt:piez?')

    def set_wavelength(self,wavelength='',waitstep=1,closeenough=0.01,timeout=60,wait=True):
        """Set wavelength in nm and track to it, if wait=True program will wait"""

        if isinstance(wavelength,float) or isinstance(wavelength,int):
            if wavelength < 1550 or wavelength > 1630: raise Exception('Wavelength out of range.')

            self.write('outp:scan:stop')
            sleep(waitstep)
            self.write('outp:trac 1') #needs wavelength tracking activated
            sleep(waitstep)
            if float(self.query('sour:wave?')) != wavelength:
                self.write('sour:wave {}'.format(wavelength))

            if wait:
                time0 = time()
                goodtime = time()
                flag = False
                while (time() - goodtime < 3) or (abs(float(self.query('sens:wave?'))-wavelength) > closeenough):
                    if flag == False:
                        goodtime = time()
                    if abs(float(self.query('sens:wave?'))-wavelength) < closeenough and flag == False:
                        flag = True
                        goodtime = time()
                    if time() - time0 > timeout:
                        raise Exception('{} sec timeout reached whilst waiting for wavelength {}.'.format(timeout,wavelength))
                    sleep(waitstep)
            self.write('outp:trac 0')
        else:
            return self.query('sour:wave?')

    def read_wavelength(self):
        """Return instantaneous wavelength in nm"""
        return self.query('sens:wave?')

    def tracking(self,trac=''):
        """Set tracking"""
        if isinstance(trac,bool) or isinstance(trac,int):
            self.write('outp:trac {}'.format(int(trac)))
        else:
            return self.query('outp:trac?')

    #Sweeps
    def setup_sweep(self,startwave=1520,stopwave=1570,speed=20,returnspeed='auto',scans=1):
        """Set up a sweep, wavelength in nm, speed in nm/s"""
        self.write('outp:scan:stop') #stop
        self.write('sour:wave:scancfg 0') #keep laser on during reverse sweep
        self.write('sour:wave:desscans {}'.format(scans))
        self.write('sour:wave:start {}'.format(startwave))
        self.write('sour:wave:stop {}'.format(stopwave))

        if speed > 20: raise Exception('Laser speed {} too fast.'.format(speed))
        self.write('sour:wave:slew:forw {}'.format(speed))

        if isinstance(returnspeed,int) or isinstance(returnspeed,float):
            if returnspeed > 20: raise Exception('Laser return speed {} too fast.'.format(returnspeed))
            self.write('sour:wave:slew:ret {}'.format(returnspeed))
        else:
            self.write('sour:wave:slew:ret {}'.format(speed))

    def cont_sweep(self,startwave=1520,stopwave=1570,speed=20,returnspeed='auto',scans=1):
        """Continuously sweep laser, wavelength in nm, speed in nm/s"""
        self.setup_sweep(startwave=startwave,stopwave=stopwave,speed=speed,returnspeed=returnspeed,scans=scans)
        sleep(1)
        self.write('outp:scan:start')

    def start_sweep(self,check=True):
        """Start sweep"""
        if check and (self.query('*OPC?') == '0'):
            raise Exception('Laser not ready.')
        self.write('outp:scan:start')

    #Laser on/off
    def on_delay(self):
        """Time delay between sending on command and laser starting, converted to seconds"""
        return float(self.query('ondelay?'))/1000
    
    def on(self,wait=True):
        """Turn laser on"""
        if self.query('outp:stat?') == '0':
            self.write('outp:stat 1')
            if wait:
                sleep(self.on_delay())
            print('Laser on')
   
    def off(self):
        """Turn laser off"""
        if self.query('outp:stat?') == '1':
            self.write('outp:stat 0')
            print('Laser off')

    def close(self):
        """Disconnect communication"""
        self.write('syst:mcon loc')
        self.npusb.CloseDevices()

    #Wavelength monitoring functions
    def volt_to_nm(self,voltage):
        """Convert wavelength voltage V to nm"""
        return (1570-1520)*voltage/10+1520

    def nm_to_volt(self,wave):
        """Convert wavelength nm to V"""
        return 10*(wave-1520)/(1570-1520)


# -*- coding: utf-8 -*-
"""
Created on Thu May 26 17:25:04 2022

@author: allen
"""

from labequipment.instruments.awg.signal_generator.SG384 import SG384
import matplotlib.pyplot as plt
plt.close('all')

# ESA
SG_address = 'GPIB0::27::INSTR'

with SG384(SG_address) as sg:
    sg.tuneAmp(6)
    print(sg.query('AMPR?'))
# plt.plot(data.Frequency,data.Intensity)


import pyvisa

import pandas as pd

class AQ3675:
    def __init__(self, GPIB_address):
        self.address = GPIB_address
        self.rm = pyvisa.ResourceManager()
    def __enter__(self):
        self.osa = self.rm.open_resource(self.address)
        self.osa.timeout = 10e3
        
        return self
    def fetchData(self):
        x = self.osa.query(':TRAC:DATA:X? TRA')
        y = self.osa.query(':TRAC:DATA:Y? TRA')
        df = pd.DataFrame({'Wavelength':x.split(','),
                           'Intensity':y.split(',')})
        df = df.apply(pd.to_numeric,errors='coerce')
        
        return df
    def __exit__(self, exc_type, exc_value, traceback):
        self.osa.close()


import pyvisa

import numpy as np
import pandas as pd

class HP54810A:
    def __init__(self, GPIB_address):
        self.address = GPIB_address
        self.rm = pyvisa.ResourceManager()
    def __enter__(self):
        self.scope = self.rm.open_resource(self.address)
        self.scope.timeout = 2000
        self.scope.write(':WAVEFORM:SOURCE CHAN1') 
        self.scope.write(':TIMEBASE:MODE MAIN')
        self.scope.write(':ACQUIRE:TYPE NORMAL')
        self.scope.write(':ACQUIRE:COUNT 1')
        self.scope.write(':ACQUIRE:POINTS 5000')
        self.scope.write(':WAV:POINTS:MODE RAW')
        self.scope.write(':WAV:POINTS 5000')
        self.scope.write(':WAVEFORM:FORMAT ASCii')
        return self
    def fetchData(self):
        preambleBlock = self.scope.query(':WAVEFORM:PREAMBLE?')
        RawData = self.scope.query(':WAV:DATA?')
        RawData = RawData.replace('\r\n\x00','')
        RawData = RawData.split('\n')[0]
        RawData = [float(i) for i in RawData.split(',')]
        RawData = np.array(RawData)
        preambleBlock = preambleBlock.split(',')
        XIncrement = float(preambleBlock[4]) # in seconds
        XData = (XIncrement*(np.arange(len(RawData)))) - XIncrement
        YData = RawData

        df = pd.DataFrame({'Time':XData,
                           'Volts':YData})
        return df
    def send(self, command):
        self.scope.write(command)
    def get(self):
        msg = self.scope.read()
        return msg
    def __exit__(self, exc_type, exc_value, traceback):
        self.scope.close()



import pyvisa

import pandas as pd

class E5052A:
    def __init__(self, GPIB_address):
        self.address = GPIB_address
        self.rm = pyvisa.ResourceManager()
    def __enter__(self):
        self.ssa = self.rm.open_resource(self.address)
        return self
    def fetchData(self):
        x = self.ssa.query('CALC:PN1:DATA:XDAT?').split(',')
        y = self.ssa.query('CALC:PN1:DATA:RDAT?').split(',')
        df = pd.DataFrame({'Frequency':x,
                           'PN':y})
        df = df.apply(pd.to_numeric,errors='coerce')
        return df
    def getCarrier(self):
        carrier = self.ssa.query('CALC:PN1:DATA:CARR?')
        fc = float(carrier.split(',')[0])*1e-9
        power = float(carrier.split(',')[1])
        return (fc,power)
    def query(self,command):
        msg = self.ssa.query(command)
    def send(self, command):
        self.ssa.write(command)
    def get(self, command):
        msg = self.ssa.read(command)
        return msg
    def __exit__(self, exc_type, exc_value, traceback):
        self.ssa.close()
  


import pyvisa
import numpy as np
import pandas as pd

class E4440A:
    def __init__(self, GPIB_address='GPIB0::18::INSTR'):
        self.address = GPIB_address
        self.rm = pyvisa.ResourceManager()
    def __enter__(self):
        self.esa = self.rm.open_resource(self.address)
        return self
        
    def fetchData(self,startFreq=-1,stopFreq=-1):
        if startFreq != -1:
            self.esa.write(f'SENS:FREQ:STAR {float(startFreq) / float(1e9)} GHz')
        if stopFreq != -1:
            self.esa.write(f'SENS:FREQ:STOP {float(stopFreq) / float(1e9)} GHz')

        data=self.esa.query('TRAC:DATA? TRACE1')
        freqStart=float(self.esa.query('SENS:FREQ:STAR?'))
        freqStop=float(self.esa.query('SENS:FREQ:STOP?'))
        step=float(self.esa.query('SENS:FREQ:CENT:STEP:INCR?'))

        intensity=np.array([float(i) for i in data.split(',')])
        freq=np.linspace(freqStart,freqStop,len(intensity))
                
        df = pd.DataFrame({'Frequency':freq,
                           'Intensity':intensity})

        return df

    def __exit__(self, exc_type, exc_value, traceback):
        self.esa.close()


import pyvisa

import numpy as np
import pandas as pd

from struct import unpack


class TDS7404:
    def __init__(self, GPIB_address):
        self.address = GPIB_address
        self.rm = pyvisa.ResourceManager()
    def __enter__(self):
        self.scope = self.rm.open_resource(self.address)
        return self
    def fetchData(self, channel):
        self.scope.write(f"DATA:SOU CH{channel}")
        self.scope.write('DATA:WIDTH 1')
        self.scope.write('DATA:ENC RPB')
        ymult = float(self.scope.query('WFMPRE:YMULT?'))
        yzero = float(self.scope.query('WFMPRE:YZERO?'))
        yoff = float(self.scope.query('WFMPRE:YOFF?'))
        xincr = float(self.scope.query('WFMPRE:XINCR?'))
        xdelay = float(self.scope.query('HORizontal:POSition?'))
        self.scope.write('CURVE?')
        data = self.scope.read_raw()
        headerlen = 2 + int(data[1])
        header = data[:headerlen]
        ADC_wave = data[headerlen:-1]
        ADC_wave = np.array(unpack('%sB' % len(ADC_wave),ADC_wave))
        Volts = (ADC_wave - yoff) * ymult  + yzero
        Time = np.arange(0, (xincr * len(Volts)), xincr)-((xincr * len(Volts))/2-xdelay)
        df = pd.DataFrame({'Time':Time-np.min(Time),
                           'Volts':Volts})
        return df
    def send(self, command):
        self.scope.write(command)
    def get(self, command):
        msg = self.scope.read(command)
        return msg
    def __exit__(self, exc_type, exc_value, traceback):
        self.scope.close()




# -*- coding: utf-8 -*-
"""
Created on Fri Jul  8 08:03:57 2022

@author: allen
"""
from labequipment.instruments.ssa.E5052A import E5052A
import matplotlib.pyplot as plt
import os

import fnmatch

plt.close('all')

# OSA
SSA_address = 'GPIB0::17::INSTR'

path = '230103'
with E5052A(SSA_address) as ssa:
    data = ssa.fetchData()
    fc, power = ssa.getCarrier()
    print(f'fc: {fc:.2f}GHz, power: {power:.2f}dBm')
plt.semilogx(data.Frequency,data.PN)
plt.ylim(-140,0)
fc = str(round(fc,5)).replace('.','p')
power = str(round(power,2)).replace('.','p')
count = len(fnmatch.filter(os.listdir(path), '*.*'))

data.to_csv(os.path.join(path,f'Soliton' + str(count) + '.csv'),index=False)


# -*- coding: utf-8 -*-
"""
Created on Fri Jul  8 00:18:42 2022

@author: allen
"""
from labequipment.instruments.ssa.N9010A import N9010A
import matplotlib.pyplot as plt
import os
plt.close('all')

# EXA
EXA_address = 'GPIB0::18::INSTR'

path = '220725'
with N9010A(EXA_address) as exa:
    data = exa.fetchData()
    
    
plt.plot(data.Frequency,data.Intensity)
data.to_csv(os.path.join(path,'ESA_IQphaseShifter_source_I0_Q0.csv'),index=False)



import pyvisa

import numpy as np
import pandas as pd

class R3465:
    def __init__(self, GPIB_address, samples=1001):
        self.address = GPIB_address
        self.rm = pyvisa.ResourceManager()
        self.samples = samples
        self.maxLevel = 14592
        self.minLevel = 1792
    def __enter__(self):
        self.esa = self.rm.open_resource(self.address)
        return self
    def fetchData(self,startFreq=0,stopFreq=3e9,dbpdiv=10,reflevel=10,Navg=3):
        self.esa.write(f'FA {startFreq}')
        self.esa.write(f'FB {stopFreq}')
        self.esa.write(f'DD {dbpdiv}')
        self.esa.write(f'RL {reflevel}')
        freq = np.linspace(startFreq,stopFreq,self.samples)
        DivOut = int(self.esa.query('DD?'))
        if DivOut == 0:
            Div = 10
        elif DivOut == 1:
            Div = 5
        elif DivOut == 2:
            Div = 2
        elif DivOut == 3:
            Div = 1
        elif DivOut == 4:
            Div = 0.5
        Ref = float(self.esa.query('RL?'))
        
        data = list()
        for N in range(Navg):
            self.esa.write('TAA?')
            dataTmp = list()
            for i in range(1001):
                tmp = self.esa.read()
                tmp = ((float(tmp)-self.minLevel)/(self.maxLevel-self.minLevel)-1)*10*Div+Ref
                dataTmp.append(10**(tmp*0.1))
            data.append(dataTmp)
        data = 10*np.log10(np.mean(np.array(data),axis=0))
        df = pd.DataFrame({'Frequency':freq,
                           'Intensity':data})
        
        
        return df
    def __exit__(self, exc_type, exc_value, traceback):
        self.esa.close()



import pyvisa

class SG384:
    def __init__(self, GPIB_address):
        self.address = GPIB_address
        self.rm = pyvisa.ResourceManager()
    def __enter__(self):
        self.sg = self.rm.open_resource(self.address)
        self.sg.write('ENBR 1')
        self.sg.write('AMPR 1 vpp')
        return self
    def tuneFreq(self,freq):
        self.sg.write(f'FREQ {freq}')
    def tuneAmp(self,amp):
        self.sg.write(f'AMPR {amp}')
    def query(self,command):
        msg = self.sg.query(command)
        return msg
    def send(self,command):
        self.sg.write(command)
    def __exit__(self, exc_type, exc_value, traceback):
        self.sg.write('ENBR 0')
        self.sg.close()


# -*- coding: utf-8 -*-
"""
Created on Sat May 21 14:30:57 2022

@author: allen
"""

from labequipment.instruments.esa.R3465 import R3465
import matplotlib.pyplot as plt
import os
plt.close('all')

# ESA
ESA_address = 'GPIB0::18::INSTR'

path = '230119'
with R3465(ESA_address) as esa:
    data = esa.fetchData(startFreq=10e6,stopFreq=1e9,Navg=1)
    
    
plt.plot(data.Frequency,data.Intensity)
data.to_csv(os.path.join(path,'Soli1.csv'),index=False)


import pyvisa

import numpy as np
import pandas as pd


class N9010A:
    def __init__(self, GPIB_address):
        self.address = GPIB_address
        self.rm = pyvisa.ResourceManager()
    def __enter__(self):
        self.exa = self.rm.open_resource(self.address)
        return self
    def fetchData(self):
        data = self.exa.query('CALC:DATA?').split(',')
        data = np.array(data)
        data = data.reshape((-1,2))
        df = pd.DataFrame(data,columns=['Frequency','Intensity'])
        df = df.apply(pd.to_numeric,errors='coerce')
        return df
    def send(self, command):
        self.exa.write(command)
    def get(self, command):
        msg = self.exa.read(command)
        return msg
    def __exit__(self, exc_type, exc_value, traceback):
        self.exa.close()
      


import matplotlib.pyplot as plot
from labequipment.instruments.oscilloscopes.DS1104 import DS1104
import pandas as pd
import os

path = '230915_2'


scope = DS1104()
[time, tUnit, data1, data2, info] = scope.fetchData()
data = [time, tUnit, data1, data2, info]

plot.figure(1)
ax1 = plot.subplot(211)
plot.plot(time, data1)
plot.title("Channel 1")
plot.ylabel("Voltage (V)")
plot.xlabel("Time (" + str(tUnit) + ")")
plot.xlim(time[0], time[-1])
plot.subplots_adjust(hspace = 0.5)

plot.subplot(212, sharey=ax1)
plot.plot(time, data2)
plot.title("Channel 2")
plot.ylabel("Voltage (V)")
plot.xlabel("Time (" + str(tUnit) + ")")
plot.xlim(time[0], time[-1])

plot.show()

df = pd.DataFrame([time, data1, data2])
df.to_csv(os.path.join(path,f'Q_1056res.csv'),index=False)






# -*- coding: utf-8 -*-
"""
Created on Sat May 21 14:30:57 2022

@author: allen
"""

from labequipment.instruments.osa.AQ3675 import AQ3675
import matplotlib.pyplot as plt
import os
plt.close('all')

# OSA
OSA_address = 'GPIB0::1::INSTR'

path = '220912_LLNL'
with AQ3675(OSA_address) as osa:
    data = osa.fetchData()
    
    
plt.plot(data.Wavelength,data.Intensity)
# plt.yscale('log')
data.to_csv(os.path.join(path,'ELMA.csv'),index=False)


# -*- coding: utf-8 -*-
"""
Created on Sat Jul  9 15:09:59 2022

@author: allen
"""
from labequipment.instruments.oscilloscopes.TDS7404 import TDS7404
import matplotlib.pyplot as plt
import os
plt.close('all')

# OSC
OSC_address = 'GPIB0::3::INSTR'

path = '220709'
with TDS7404(OSC_address) as osc:
    data = osc.fetchData(channel=2)
    
plt.plot(data.Time,data.Volts)
data.to_csv(os.path.join(path,'Osc_ch2.csv'),index=False)





import time as tm
import numpy
import easy_scpi as spci


class DS1104(object):
    def __init__(self):
        self.instr = spci.Instrument("USB0::0x1AB1::0x04CE::DS1ZA203514719::INSTR")

        self.instr.connect()
    def __del__(self):
        self.instr.disconnect()
    
    def single(self):
        self.instr.write(":SING")
    def continuous(self):
        self.instr.write(":RUN")
    def stop(self):
        self.instr.write("STOP")

    def fetchData(self):
        tm.sleep(0.5)
        status = self.instr.query(":TRIG:STAT?")
        if status == "WAIT\n" or status == "RUN\n" or status == 'AUTO\n':
            self.instr.write("STOP")
            self.instr.write("RUN")
            tm.sleep(2.5)
            self.instr.write("STOP")

        self.instr.write(":WAV:MODE NORM")
        
        self.instr.write(":WAV:SOURCE CHAN1")
        self.instr.write(":WAV:MODE NORM")
        self.instr.write(":WAV:FORM ASC")
        rawdata1 = self.instr.query(":WAV:DATA?")[11:]

        self.instr.write(":WAV:SOURCE CHAN2")
        self.instr.write(":WAV:MODE NORM")
        self.instr.write(":WAV:FORM ASC")
        
        rawdata2 = self.instr.query(":WAV:DATA?")[11:]
        data_size = len(rawdata1)

        sample_rate = float(self.instr.query(":ACQ:SRAT?"))
        timescale = float(self.instr.query(":TIM:SCAL?"))
        timeoffset = float(self.instr.query(":TIM:OFFS?"))
        voltscale1 = float(self.instr.query(":CHAN1:SCAL?"))
        voltoffset1 = float(self.instr.query(":CHAN1:OFFS?"))

        voltscale2 = float(self.instr.query(":CHAN2:SCAL?"))
        voltoffset2 = float(self.instr.query(":CHAN2:OFFS?"))


        
        data1 = rawdata1.split(',')
        data1 = numpy.float64(numpy.array(data1))

        data2 = rawdata2.split(',')
        data2 = numpy.float64(numpy.array(data2))

        time = numpy.linspace(timeoffset - 6*timescale,
                              timeoffset + 6*timescale,
                              num=len(data1))
        
        if(time[-1] < 1e-3):
            time = time * 1e6
            tUnit = u"\u00B5\u0053"
        elif (time[-1] < 1):
            time = time * 1e3
            tUnit = "ms"
        else:
            tUnit = "s"
        
        info = [("Data Size", data_size),
                ("Sample Rate", sample_rate),
                ("Time Scale", timescale),
                ("Voltage Offset CH1", voltoffset1),
                ("Voltage Scale CH1", voltscale1),
                ("Voltage Offset CH2", voltoffset2),
                ("Voltage Scale CH2", voltscale2),
                ("Time Unit", tUnit)]
        #self.instr.write(":RUN")
        #self.instr.write(":KEY:FORC")

        return [time, tUnit, data1, data2, info]

